<?php
/**
 * VibeForms API
 *
 * @author 		VibeThemes
 * @category 	Init
 * @package 	vibeforms/Includes
 * @version     1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VibeForms_Api{

	public static $instance;
    
    public static function init(){

        if ( is_null( self::$instance ) )
            self::$instance = new VibeForms_Api();
        return self::$instance;
    }

	private function __construct(){
		add_action('rest_api_init',array($this,'register_api_endpoints'));
	}


	function register_api_endpoints(){

		register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_forms' ),
                'permission_callback' => array( $this, 'set_user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/uploadfile', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'uploadfile' ),
                'permission_callback' => array( $this, 'user_permissions_upload_check' ),
            ),
        ));

                

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/types', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_form_types' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));


        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/new', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'new_form' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/delete', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'delete_form' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/submit', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'submit_form' ),
                'permission_callback' => array( $this, 'set_user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/can_submit', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'can_submit_form' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/submissions', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_all_submissions' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        //single form id based
        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/form/submissions', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'get_form_submissions' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/addShared', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'add_shared' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));
        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/removeShared', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'remove_shared' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));
        register_rest_route( VIBEFORMS_API_NAMESPACE, '/user/forms/trash', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'trash_form' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));

        register_rest_route( VIBEFORMS_API_NAMESPACE, '/users/search/results', array(
            array(
                'methods'             =>  'POST',
                'callback'            =>  array( $this, 'search_results' ),
                'permission_callback' => array( $this, 'user_permissions_check' ),
            ),
        ));
    }

    function set_user_permissions_check($request){
        $body = json_decode($request->get_body(),true);
        if(!empty($body['token'])){
            global $wpdb;
            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
        }
        return true;
    }

    function user_permissions_upload_check($request){
        $body = $_POST['body'];
        $body  = json_decode(stripslashes($body),true);
        if(!empty($body['token'])){

            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
                return true;
            }
        }
        return false;
    }
    function user_permissions_check($request){
        $body = json_decode($request->get_body(),true);
        if(!empty($body['token'])){

            $this->user = apply_filters('vibebp_api_get_user_from_token','',$body['token']);
            if(!empty($this->user)){
                return true;
            }
        }
        return false;
    }

    function get_forms($request){
        $args = json_decode($request->get_body(),true);

        $form_args = array(
            'post_type'=>'vibeform',
            'posts_per_page'=>empty($args['posts_per_page'])?20:$args['posts_per_page'],
            'paged'=>empty($args['paged'])?1:$args['paged'],
            's'=>empty($args['s'])?'':$args['s'],
            'order' => empty($args['order'])?'DESC':$args['order'],
            'orderby' => empty($args['orderby'])?'title':$args['orderby'],
            'fields'        => 'ids',
        );

        if(!empty($args['embed_post_id'])){
            if(get_post_meta($args['embed_post_id'],'embed',true)){
                $form_args['post__in'] = array($args['embed_post_id']);
            }else{
                return new WP_REST_Response(array('status' => 0 ,'message' =>__('Public access to form is disabled!','vibeforms')), 200);
            }
        }

        if(!empty($args['embed'])){
            $form_args['author']=$this->user->id;
            $form_args['meta_query'][]=array(
                'key'=>'embed',
                'value'=>'1',
                'compare'=>'='
            );
        }

        

        if(!empty($args['form-type'])){
            $form_args['tax_query']=array(
                array(
                    'taxonomy'=>'vibeform-type',
                    'field'=>'term_id',
                    'terms'=>$args['form-type'],
                ),
            );
        }
        if(!empty($args['type'])){
            switch($args['type']){
                case 'mine':
                    $form_args['author']=$this->user->id;
                break;
                case 'shared':
                    $posts = array();
                    // group/user shared
                    if(!empty($args['shared_type'])){
                        $posts = vibeforms_get_user_shared_forms($this->user->id,$args['shared_type']);
                    }else{
                        $posts = vibeforms_get_user_shared_forms($this->user->id);
                    }
                    if(!empty($posts)){
                        $form_args['post__in']=$posts;
                    }else{
                        $form_args['post__in']=array(0);
                    }
                    
                break;
            }
        }
        

        $return = array( 'status'=>0);
        if(!empty($this->user)){
            $forms_query = new WP_Query(apply_filters('vibeforms_get_forms_api',$form_args,$this->user));
        }else{
            $forms_query = new WP_Query(apply_filters('vibeforms_get_forms_api',$form_args));
        }
        
        $forms = array();
        while($forms_query->have_posts()){
            $forms_query->the_post();
            global $post;
            $id = get_the_ID();

            //if type not set remove all shared things
            $type = get_post_meta($id,'vibeform_shared_type',true);
            if(empty($type)){
                $type = 'user';
                delete_post_meta($id,'vibeform_shared');
                update_post_meta($id,'vibeform_shared_type',$type);
            }

            $f=array(
                'id'=>$id,
                'post_title'=>get_the_title(),
                'post_date'=>strtotime(get_post_field('post_date',$id)),
                'post_content'=>maybe_unserialize(get_the_content()),
                'post_author'=>get_the_author_ID(),
                'taxonomy'=>vibe_get_post_terms_fields($id),
                'submissions_count'=>get_comments_number($id),
                'meta'=>array(
                    'shared'=>array(
                        'type' => $type,
                        'ids' => vibeforms_get_form_shared($id)  
                    ),
                    'embed' => get_post_meta($id,'embed',true),
                    'options' => vibe_form_get_meta_options($id)
                )
            );
            if(!empty($f['post_content']['fields'])){
                foreach($f['post_content']['fields'] as $key => $field){
                    if(!empty($field['value']) && !empty($field['value']['allowed_types'])){
                        foreach($field['value']['allowed_types'] as $type){
                            if(!empty(vibe_forms_get_mime_types()[$type])){
                                if(!isset($f['post_content']['fields'][$key]['value']['allowed_mime_types'])){
                                    $f['post_content']['fields'][$key]['value']['allowed_mime_types'] = [];
                                }
                                $new_types = [];
                                if(is_array(vibe_forms_get_mime_types()[$type])){
                                    $new_types = vibe_forms_get_mime_types()[$type];
                                }else{
                                    $new_types = [vibe_forms_get_mime_types()[$type]];
                                }
                                $f['post_content']['fields'][$key]['value']['allowed_mime_types'] = array_merge($f['post_content']['fields'][$key]['value']['allowed_mime_types'],$new_types); 
                            }
                        }

                    }
                }
            }

            $forms[] = $f;
            
        }
        wp_reset_postdata();
        if(!empty($forms)){
            $return = array(
                'status'=>1,
                'forms'=>$forms,
                'total'=>$forms_query->found_posts
            );    
        }
        
       

        return new WP_REST_Response($return, 200);
    }



    function get_form_types($request){
        if(empty($this->user)){
            new WP_REST_Response(array(), 401);
        }

        $terms = get_terms( array(
            'taxonomy' => 'vibeform-type',
            'hide_empty' => false,
        ) );

        // creating required fields
        $arr = array();
        foreach ($terms as $key => $value) {
            $arr[] = array( 'term_id' => $value->term_id, 'name' => $value->name );
        }

        return new WP_REST_Response(array('status'=>1,'types'=>$arr), 200);

    }

    function new_form($request){
        $body = json_decode($request->get_body(),true);
        $args = $body['post'];
       
        if(!empty($args['post_title']) && !empty($args['post_content'])){

            if(is_serialized(maybe_serialize($args['post_content']))){
                $form_args = array(
                    'post_type'=>'vibeform',
                    'post_status'=>'publish',
                    'post_title'=>$args['post_title'],
                    'post_author'=>$this->user->id,
                    'post_content' => maybe_serialize($args['post_content']),
                    'meta'=>!empty($args['meta'])?$args['meta']:[],
                    'taxonomy' => !empty($args['taxonomy'])?$args['taxonomy']:[],
                );
                if(!empty($args['id'])){
                    if(!$this->can_edit($args['id'])){
                        return new WP_REST_Response(array('status'=>0,'message'=>__('Article author does not match.','vibeforms')), 200);
                    }
                    $form_args['ID'] = intval($args['id']);
                    $post_id = $form_args['ID'];
                    $update = wp_update_post($form_args);
                    if(empty($update)){
                        return new WP_REST_Response(array('status'=>0,'message'=>__('Article could not be updated.','vibeforms')),200);
                    }
                    $form_args['id'] = intval($args['id']);
                    unset($form_args['ID']);
                }else{
                    $post_id = wp_insert_post($form_args);
                    if(empty($post_id)){
                        return new WP_REST_Response(array('status'=>0,'message'=>__('Article could not be created.','vibeforms')),200);
                    }
                    
                    $form_args['id'] = $post_id;
                }

                //assign taxonomy to post_id
                if(isset($args['taxonomy'])){   
                    $ids = array();
                    if(is_array($args['taxonomy']) && count($args['taxonomy'])){
                        foreach ($args['taxonomy'] as $key => $value) {
                            if(!empty($value['value'])){
                                $ids[]  = (int)$value['value'];
                            }
                        }
                    }
                    wp_set_post_terms($post_id,$ids,'vibeform-type');
                }
                $form_args['post_content'] = maybe_unserialize($form_args['post_content']);
                if(!empty($args['meta'])){
                    if(isset($args['meta']['shared'])){   
                       delete_post_meta($post_id,'vibeform_shared');
                       if(!empty($args['meta']['shared']['type'])){
                           update_post_meta($post_id,'vibeform_shared_type',$args['meta']['shared']['type']) ;
                           if(count($args['meta']['shared']['ids'])){
                                foreach ($args['meta']['shared']['ids'] as $id) {
                                    add_post_meta($post_id,'vibeform_shared',(int)$id);
                                }
                            }
                        } 
                    }
                    if(isset($args['meta']['options']) && is_array($args['meta']['options'])){   
                        foreach ($args['meta']['options'] as $key => $value) {
                            update_post_meta($post_id,$key,$value);
                        }
                    }
                    if(isset($args['meta']['embed'])){   
                        update_post_meta($post_id,'embed',$args['meta']['embed']); 
                    }
                }
            }else{
                return new WP_REST_Response(array('status'=>0,'message'=>__('Form data invalid','vibeforms')),200);
            }
        }else{
            return new WP_REST_Response(array('status'=>0,'message'=>__('Data missing!','vibeforms')), 200);
        }
        return new WP_REST_Response(array('status'=>1,'form'=>$form_args,'followermessage'=>sprintf(__('%s published a new form %s','vibeforms'),$this->user->displayname,get_the_title($form_args['id']))), 200);
    }

    function delete_form($request){
        $body = json_decode($request->get_body(),true);
        $id = $body['id'];
        if(!empty($id)){
            if(!$this->can_edit($id)){
                return new WP_REST_Response(array('status'=>0,'message'=>__('Form author can delete forms.','vibeforms')), 200);
            }
            if(get_post_type($id) == 'vibeform'){
                wp_trash_post($id); 
                return new WP_REST_Response(array('status'=>1,'message'=>sprintf(__('Form %s deleted.','vibeforms'),$args['post_title'])), 200);   
            }
            
        }
        return new WP_REST_Response(array('status'=>0,'message'=>__('Can not delete form !','vibeforms')), 200);

    }

    function add_shared($request){
        $args = json_decode($request->get_body(),true);
        add_user_meta($args['user_id'],'vibeform_shared',$args['form_id']);
        return new WP_REST_Response(array('status'=>1,'message'=>__('Article shared with user','vibeforms'),'recievingmessage'=>sprintf(__('%s shared an form %s with you','vibeforms'),$this->user->display_name,get_the_title($args['form_id']))), 200);
    }

    function remove_shared($request){
        $args = json_decode($request->get_body(),true);
        remove_user_meta($args['user_id'],'vibeform_shared',$args['form_id']);
        return new WP_REST_Response(array('status'=>1,'message'=>__('Form sharing removed from user','vibeforms')), 200);
    }

    function trash_form($request){
        $args = json_decode($request->get_body(),true);
        if($this->can_edit($args['form_id'])){
            if(wp_trash_post($args['form_id'])){
                return new WP_REST_Response(array('status'=>1,'message'=>__('Form moved to trash','vibeforms')), 200);
            }else{
                return new WP_REST_Response(array('status'=>1,'message'=>__('Form could not be moved to trash!','vibeforms')), 200);
            }
        }
    }

    function can_edit($post_id){
        return $this->user->id == get_post_field('post_author',$post_id);
    }

    function submit_form($request){
        $args = json_decode($request->get_body(),true);

        $post_id = $args['id'];
        $form_data = $args['form_data'];
        $data = array();
        if(!empty($post_id) && !empty($form_data)){

            if(!empty($this->user)){
                $user_id = $this->user->id;
            }else{
                return new WP_REST_Response(array('status'=>0,'message'=>__('You need to log-in first!','vibeforms')), 200);
            }

            $arr = array(
                'comment_post_ID' => $post_id,
                'comment_type' => 'form_submission',
                'user_id' => $user_id
            );
            
            $check = $this->check_can_user_submit_form($post_id);
            if($check['status']){ //if embed then pass the checks
                $arr['comment_content'] = maybe_serialize($form_data);
                $comment_id = wp_insert_comment($arr);
                if(!empty($comment_id)){
                    $comment = get_comment($comment_id,'ARRAY_A');
                    $comment['comment_content'] = unserialize($comment['comment_content']);
                    $data = array( 'status'=>1, 'message'=>__('Form submitted','vibeforms') , 'data'=>$comment);
                    do_action('vibe_forms_submitted',$post_id, $this->user->id, $arr, $comment);
                }else{
                    $data = array( 'status'=>0, 'message'=>__('Form not submitted','vibeforms'));
                }

            }else{
                $data = $check;
            }   
        }else{
            $data = array('status'=>0,'message'=>__('Data missing!','vibeforms'));
        }
        $data = apply_filters('vibe_submit_form',$data,$request);
        return new WP_REST_Response($data, 200);
    }

    function can_submit_form($request){
        $body = json_decode($request->get_body(),true);
        $post_id = $body['post_id'];
        if(!empty($post_id)){
            $data = $this->check_can_user_submit_form($post_id);
        }else{
            $data = array('status'=>0,'message'=>__('Data missing!','vibeforms'));
        }
        return new WP_REST_Response($data, 200);
    }

    // check using form meta>options
    function check_can_user_submit_form($post_id){
        $data = array('status'=>1,'message'=>__('User can submit the form','vibeforms'));
        $options =  vibe_form_get_meta_options($post_id);
        if(isset($options['submission_last_date'])){
            if(!(time()*1000<=$options['submission_last_date'])){
                return array('status'=>0,'message'=>__('Time reached for submission','vibeforms'));
            }
        }
        if(isset($options['submission_times'])){
            if($options['submission_times'] >= 0){
                if($options['submission_times'] == 0){
                    return array('status'=>0,'message'=>__('Submission is disabled','vibeforms'));
                }else{
                    global $wpdb;
                    $sub = $wpdb->get_results( "SELECT comment_id FROM {$wpdb->comments} WHERE  comment_type='form_submission' AND comment_post_ID = {$post_id} AND user_id={$this->user->id} AND comment_approved=1",ARRAY_A);
                    if(!(count($sub)<$options['submission_times'])){
                        return array('status'=>0,'message'=>__('Already submitted the form','vibeforms'));
                    }
                }
            } 
        }
        
        return $data;
    }


    function get_form_submissions($request){
        $body = json_decode($request->get_body(),true);
        $post_id = $body['post_id'];
        $args = $body['args'];
        if(!empty($post_id) && !empty($args) && !empty($args['type'])){
            $comment_args = array(
                'paged'=> empty($args['paged'])?1:$args['paged'],
                'number' => 50,
                'post__in' => array($post_id),
                'comment_type' => 'form_submission',
                'fields' => 'ids',
                'order' => empty($args['order'])?'DESC':$args['order']
            );
            switch ($args['type']) {
                case 'mine':
                        $comment_args['user_id'] = $this->user->id;
                    break;
                case 'all':
                        if(!$this->can_edit($post_id)){
                            $data = array('status'=>0,'message'=>__('No access to view form submissions!','vibeforms'));
                            return new WP_REST_Response($data, 200);
                        }
                    break;   
                default:
                        $comment_args['user_id'] = $this->user->id;
                    break;
            }

           
            $comments_obj = new WP_Comment_Query($comment_args);
            $comment_IDs = $comments_obj->comments;
            
            if(!empty($comment_IDs)){
                $submissions = [];
                foreach ($comment_IDs as $key=> $comment_id) {
                    $submissions[] = $this->get_submission_fields($comment_id);
                }
                $data = array(
                    'status'=>1,
                    'message'=>__('Submissions found','vibeforms'),
                    'data' => $submissions
                );
            }else{
                $data = array('status'=>0,'message'=>__('No submissions found','vibeforms'));
            }
        }else{
            $data = array('status'=>0,'message'=>__('Data missing!','vibeforms'));
        }
        $data = apply_filters('vibe_form_get_submissions',$data,$request);
        return new WP_REST_Response($data, 200);
    }

    

    function get_form_fields($id){
        return array(
            'post_title' => get_post_field('post_title',$id),
            'post_content' => unserialize(get_post_field('post_content',$id))
        );
    }

    function get_submission_fields($id){
        $comment = get_comment($id,'ARRAY_A');
        $comment['comment_content'] =  unserialize($comment['comment_content']);
        return array(
            'comment' => $comment,
            'user' => $this->get_user_by_id($comment['user_id'])
        );
    }

    function get_user_by_id($id){
        if(empty($this->users[$id])){
            $this->users[$id] = array(
                'user_id'=>$id,
                'nickname'=>bp_core_get_user_displayname($id),
                'image'=>get_avatar_url($id)
            );
        }
        return $this->users[$id];
    }

    function search_results($request){
        $body = json_decode($request->get_body(),true);
        $search = $body['s'];
        $type = $body['type'];
        if(!empty($search) && !empty($type)){
            $data = array('status'=>0,'message'=>__('Not found','vibeforms'));
            switch ($type) {
                case 'user':
                    global $wpdb;
                    $results = $wpdb->get_results( "SELECT ID,display_name FROM {$wpdb->users} WHERE `user_nicename` LIKE '%{$search}%' OR 
                    `user_email` LIKE '%{$search}%' OR `user_login` LIKE '%{$search}%' OR `display_name` LIKE '%{$search}%'", ARRAY_A );
                    if(!empty($results)){
                        $data['status']=1;
                        foreach($results as $result){
                            $data['results'][]=array('id'=>(int)$result['ID'],'label'=>$result['display_name']);
                        }
                        $data['message']=__('Users found','vibeforms');
                    }
                    break;
                case 'group':
                    if(function_exists('bp_is_active') && bp_is_active('groups')){
                        global $wpdb, $bp;
                        $results = $wpdb->get_results( "SELECT id,name FROM {$bp->groups->table_name} WHERE `name` LIKE '%{$search}%' OR 
                        `slug` LIKE '%{$search}%'", ARRAY_A );
                        if(!empty($results)){
                            $data['status']=1;
                            foreach($results as $result){
                                $data['results'][]=array('id'=>(int)$result['id'],'label'=>$result['name']);
                            }
                            $data['message']=__('Groups found','vibeforms');
                        }
                    }
                    break;
                default:
                    break;
            }    
        }else{
            $data = array('status'=>0,'message'=>__('Data missing!','vibeforms'));
        }
        $data = apply_filters('vibe_forms_search_results',$data,$request);
        return new WP_REST_Response($data, 200);
    }

    function uploadfile($request){
        
        $body = $_POST['body'];
        $body  = json_decode(stripslashes($body),true);
        $form_id = $body['form_id'];
        $status = 0;
        $movefile = [];
        $attachment = [];

        $file_type = $body['file_type'];
        
        if(is_numeric($form_id)){
            if(!empty($_FILES)){
                if ( ! function_exists( 'wp_handle_upload' ) ) {
                    require_once( ABSPATH . 'wp-admin/includes/file.php' );
                }
                $upload_overrides = array(
                    'test_form' => false
                );
                $uploadedfiles = $_FILES['file'];
                $movefile = wp_handle_upload( $uploadedfiles, $upload_overrides );
                if ( $movefile && ! isset( $movefile['error'] ) ) {
                    $attachment=array('type'=>$file_type,'value'=>$movefile['url'], 'name'=>basename($movefile['url']));
                    return new WP_REST_Response(array('status'=>1,'message'=>_x('File uploaded successfully','','vibe-jm'),'data'=>$attachment),200);
                }
            }
        }
        if(empty($movefile['url'])){
             return new WP_REST_Response(array('status'=>0,'message'=>_x('Something went wrong!','','vibe-jm')),200);
        }
    }
}

VibeForms_Api::init();